@@ -2,7 +2,7 @@
 #define _LIBACPI_H_
 
 
-#define LIBACPI_VER "0.92"
+#define LIBACPI_VER "0.95"
 
 /* Here because we need it for definitions in this file . . . */
 #define MAX_NAME 128
@@ -20,27 +20,29 @@ typedef enum {
     BATT,
     PS_ERR,
 } power_state_t;
-    
+
 typedef enum {
     HIGH,
     MED,
     LOW,
     CRIT,
-    HARD_CRIT,
     BS_ERR,
 } batt_state_t;
 
 typedef enum {
     CHARGE,
     DISCHARGE,
+    FULL,
+    NO_CHARGE,
     CH_ERR,
 } charge_state_t;
 
 typedef enum {
-    OK,
-    CRITICAL,
-    CS_ERR,
-} cap_state_t;
+    SYSFS_CAPA_ENERGY,
+    SYSFS_CAPA_CHARGE,
+    SYSFS_CAPA_PERCENT,
+    SYSFS_CAPA_ERR,
+} sysfs_capa_t;
 
 typedef struct {
     /* general info */
@@ -48,12 +50,13 @@ typedef struct {
     /* these two are conveniences */
     char info_file[MAX_NAME];
     char state_file[MAX_NAME];
-    int present; 
+    /* sysfs capacity mode */
+    sysfs_capa_t sysfs_capa_mode;
+    int present;
     int design_cap;		/* assuming mAh */
     int last_full_cap;
     int design_voltage;		/* in mV */
     /* state info */
-    cap_state_t capacity_state;
     charge_state_t charge_state;
     int present_rate;		/* in mAh */
     int remaining_cap;		/* in mAh */
@@ -65,9 +68,9 @@ typedef struct {
     /* and a flag to indicate that this is valid . . . */
     int valid;
     /* number of times we've gotten bad info on this battery's present rate */
-    int bad_count;		
+    int bad_count;
 } battery_t;
-    
+
 typedef struct {
     char *name;
     char state_file[MAX_NAME];
@@ -93,11 +96,11 @@ typedef struct {
 
 /*
  * Moving percentage to the battery is right, but I think we need a global
- * remaining capacity somewhere, too . . . 
+ * remaining capacity somewhere, too . . .
  */
 
 /*
- * To provide a convenient debugging function . . . 
+ * To provide a convenient debugging function . . .
  *
  * It's a macro because I'm too lazy to deal with varargs.
  */
@@ -122,11 +125,11 @@ typedef struct {
 
 #define pfatal(fmt, arg...)				\
     fprintf(stderr, fmt, ##arg)				\
-	
+
 
 /* Since these /are/ needed here . . . */
-battery_t batteries[MAXBATT];
-int verbosity;
+extern battery_t batteries[MAXBATT];
+extern int verbosity;
 
 /* check if apm/acpi is enabled, etc */
 int power_init(global_t *globals);
