@@ -13,18 +13,18 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #define _GNU_SOURCE
 
-#include <dockapp.h>
+#include <libdockapp/dockapp.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <getopt.h>
 #include <unistd.h>
 #include <time.h>
 
@@ -35,9 +35,6 @@
 #include <X11/xpm.h>
 
 #include "libacpi.h"
-#include "wmacpi.h"
-
-#define WMACPI_VER "2.1"
 
 /* main pixmap */
 #ifdef LOW_COLOR
@@ -47,7 +44,13 @@ static char **master_xpm = master_low_xpm;
 #include "master.xpm"
 #endif
 
+static int battery_no;
+
+/* Do NOT change the BASE_PERIOD without reading the code ... */
+#define BASE_PERIOD 100000 /* base period, 100 ms (in usecs) */
+
 struct dockapp {
+    int x_fd;                   /* X11 fd */
     Display *display;		/* display */
     Window win;			/* main window */
     Pixmap pixmap;		/* main pixmap */
@@ -62,6 +65,8 @@ struct dockapp {
     int bell;			/* bell on critical low, or not? */
     int scroll;			/* scroll message text? */
     int scroll_reset;		/* reset the scrolling text */
+    int percent;
+    int period_length;         /* length of the polling period, multiple of BASE_PERIOD */
 };
 
 /* globals */
@@ -74,11 +79,11 @@ struct dockapp *dockapp;
 int scroll_reset = DEFAULT_SCROLL_RESET;
 
 /* copy a chunk of pixmap around the app */
-static void copy_xpm_area(int x, int y, int w, int h, int dx, int dy)					
-{										
-    XCopyArea(DADisplay, dockapp->pixmap, dockapp->pixmap,	
-	      DAGC, x, y, w, h, dx, dy);			
-    dockapp->update = 1;							
+static void copy_xpm_area(int x, int y, int w, int h, int dx, int dy)
+{
+    XCopyArea(DADisplay, dockapp->pixmap, dockapp->pixmap,
+	      DAGC, x, y, w, h, dx, dy);
+    dockapp->update = 1;
 }
 
 /* display AC power symbol */
@@ -114,7 +119,7 @@ static void clear_time_display(void)
 /* set time display to -- -- */
 static void invalid_time_display(void)
 {
-    copy_xpm_area(122, 13, 31, 11, 7, 32);
+    copy_xpm_area(122, 14, 31, 11, 7, 32);
 }
 
 static void reset_scroll(void) {
@@ -136,9 +141,13 @@ static void redraw_window(void)
 
 static void new_window(char *display, char *name, int argc, char **argv)
 {
+    XSizeHints *hints;
+
     /* Initialise the dockapp window and appicon */
-    DAInitialize(display, name, 64, 64, argc, argv);
+    DAOpenDisplay(display, argc, argv);
+    DACreateIcon(name, 64, 64, argc, argv);
     dockapp->display = DADisplay;
+    dockapp->x_fd = XConnectionNumber(dockapp->display);
     dockapp->win = DAWindow;
 
     XSelectInput(dockapp->display, dockapp->win,
@@ -146,7 +155,7 @@ static void new_window(char *display, char *name, int argc, char **argv)
                  StructureNotifyMask);
 
     /* create the main pixmap . . . */
-    DAMakePixmapFromData(master_xpm, &dockapp->pixmap, &dockapp->mask, 
+    DAMakePixmapFromData(master_xpm, &dockapp->pixmap, &dockapp->mask,
 			 &dockapp->width, &dockapp->height);
     DASetPixmap(dockapp->pixmap);
     DASetShape(dockapp->mask);
@@ -159,6 +168,20 @@ static void new_window(char *display, char *name, int argc, char **argv)
 	pfatal("FATAL: Cannot create text scroll pixmap!\n");
 	exit(1);
     }
+
+    /* force the window to stay this size - otherwise the user could
+     * resize us and see our panties^Wmaster pixmap . . . */
+    hints = XAllocSizeHints();
+    if(hints) {
+	hints->flags |= PMinSize | PMaxSize;
+	hints->min_width = 64;
+	hints->max_width = 64;
+	hints->min_height = 64;
+	hints->max_height = 64;
+	XSetWMNormalHints(dockapp->display, dockapp->win, hints);
+	XFree(hints);
+    }
+
     DAShow();
 }
 
@@ -179,15 +202,15 @@ static void scroll_text(void)
     static int start, end, stop;
     int x = 6;			/* x coord of the start of the text area */
     int y = 50;			/* y coord */
-    int width = 52;		/* width of the text area */
+    int width = 51;		/* width of the text area */
     int height = 7;		/* height of the text area */
     int tw = dockapp->tw;	/* width of the rendered text */
     int sx, dx, w;
 
-    if (!dockapp->scroll) 
+    if (!dockapp->scroll)
 	return;
 
-    /* 
+    /*
      * Conceptually this is viewing the text through a scrolling
      * window - the window starts out with the end immediately before
      * the text, and stops when the start of the window is immediately
@@ -200,9 +223,9 @@ static void scroll_text(void)
      * text is being copied, until a full window is being copied.
      *
      * As the end of the window moves out past the end of the text, we
-     * want to keep the destination at the beginning of the text area, 
+     * want to keep the destination at the beginning of the text area,
      * but copy a smaller and smaller chunk of the text. Eventually the
-     * start of the window will scroll past the end of the text, at 
+     * start of the window will scroll past the end of the text, at
      * which point we stop doing any work and wait to be reset.
      */
 
@@ -222,7 +245,7 @@ static void scroll_text(void)
 	w = end;
     else if (end > tw)
 	w = 52 - (end - tw);
-	
+
     dx = x + 52 - w;
     if (end > tw)
 	dx = x;
@@ -282,10 +305,18 @@ static void render_text(char *string)
     scroll_text();
 }
 
+static void clear_percentage(void)
+{
+  /* clear the number */
+  copy_xpm_area(95, 47, 21, 9, 37, 16);
+  /* clear the bar */
+  copy_xpm_area(66, 18, 54, 8, 5, 5);
+
+  dockapp->percent = -1;
+}
+
 static void display_percentage(int percent)
 {
-    static int op = -1;
-    static unsigned int obar;
     unsigned int bar;
     int width = 54;		/* width of the bar */
     float ratio = 100.0/width;	/* ratio between the current percentage
@@ -294,7 +325,7 @@ static void display_percentage(int percent)
     if (percent == -1)
 	percent = 0;
 
-    if (op == percent)
+    if (dockapp->percent == percent)
 	return;
 
     if (percent < 0)
@@ -302,6 +333,9 @@ static void display_percentage(int percent)
     if (percent > 100)
 	percent = 100;
 
+    if (dockapp->percent == -1)
+	copy_xpm_area(127, 28, 5, 7, 52, 17);
+
     if (percent < 100) {	/* 0 - 99 */
 	copy_xpm_area(95, 48, 8, 7, 37, 17);
 	if (percent >= 10)
@@ -309,17 +343,13 @@ static void display_percentage(int percent)
 	copy_xpm_area((percent % 10) * 6 + 67, 28, 5, 7, 46, 17);
     } else
 	copy_xpm_area(95, 37, 21, 9, 37, 16);	/* 100% */
-    op = percent;
+    dockapp->percent = percent;
 
     bar = (int)((float)percent / ratio);
 
-    if (bar == obar)
-	return;
-
     copy_xpm_area(66, 0, bar, 8, 5, 5);
     if (bar < 54)
 	copy_xpm_area(66 + bar, 18, 54 - bar, 8, bar + 5, 5);
-    obar = bar;
 }
 
 static void display_time(int minutes)
@@ -337,7 +367,7 @@ static void display_time(int minutes)
     hour = minutes / 60;
     /* our display area only fits %2d:%2d, so we need to make sure
      * what we're displaying will fit in those constraints. I don't
-     * think we're likely to see any batteries that do more than 
+     * think we're likely to see any batteries that do more than
      * 100 hours any time soon, so it's fairly safe. */
     if (hour >= 100) {
 	hour = 99;
@@ -361,7 +391,7 @@ static void display_time(int minutes)
     omin = min;
 }
 
-/* 
+/*
  * The reworked state handling stuff.
  */
 
@@ -376,13 +406,14 @@ static void really_blink_power_glyph(void)
 {
     static int counter = 0;
 
-    if (counter == 10) 
+    if (counter == 10)
 	display_power_glyph();
-    else if (counter == 20) 
+    else if (counter == 20)
 	kill_power_glyph();
     else if (counter > 30)
 	counter = 0;
-    counter++;
+
+    counter += dockapp->period_length;
 }
 
 static void blink_power_glyph(void)
@@ -401,8 +432,9 @@ static void really_blink_battery_glyph(void)
 	kill_battery_glyph();
     else if (counter > 30)
 	counter = 0;
-    counter++;
-}    
+
+    counter += dockapp->period_length;
+}
 
 static void blink_battery_glyph(void)
 {
@@ -412,7 +444,7 @@ static void blink_battery_glyph(void)
 
 static void set_power_panel(global_t *globals)
 {
-    enum panel_states power = PS_NULL;
+    static enum panel_states power = PS_NULL;
     battery_t *binfo = globals->binfo;
     adapter_t *ap = &globals->adapter;
 
@@ -430,19 +462,12 @@ static void set_power_panel(global_t *globals)
 	}
     }
 
-    if (binfo->charge_state == CHARGE)
-	blink_power_glyph();
+    if (globals->battery_count > 0) {
+	if (binfo->charge_state == CHARGE)
+	    blink_power_glyph();
 
-    if ((binfo->state == CRIT) && (ap->power == BATT))
-	blink_battery_glyph();
-
-    if (binfo->state == HARD_CRIT) {
-	really_blink_battery_glyph();
-	/* we only do this here because it'd be obnoxious to 
-	 * do it anywhere else. */
-	if (dockapp->bell) {
-	    XBell(dockapp->display, 100);
-	}
+	if ((binfo->state == CRIT) && (ap->power == BATT))
+	    blink_battery_glyph();
     }
 }
 
@@ -458,7 +483,7 @@ void reset_scroll_speed(void) {
     scroll_reset = DEFAULT_SCROLL_RESET;
 }
 
-/* 
+/*
  * The message that needs to be displayed needs to be decided
  * according to a heirarchy: a message like not present needs to take
  * precedence over a global thing like the current power status, and
@@ -466,13 +491,14 @@ void reset_scroll_speed(void) {
  * the "on battery" message. Likewise, a battery charging message
  * needs to take precedence over the on ac power message. The other
  * question is how much of a precedence local messages should take
- * over global ones . . . 
+ * over global ones . . .
  *
  * So, there are three possible sets of messages: not present, on-line
  * and off-line messages. We need to decide which of those sets is
- * appropriate right now, and then decide within them. 
+ * appropriate right now, and then decide within them.
  */
 enum messages {
+    M_NB,	/* no batteries */
     M_NP,	/* not present */
     M_AC,	/* on ac power */
     M_CH,	/* battery charging */
@@ -488,7 +514,17 @@ static void set_message(global_t *globals)
     static enum messages state = M_NULL;
     battery_t *binfo = globals->binfo;
     adapter_t *ap = &globals->adapter;
-    
+
+    if (globals->battery_count == 0) {
+	if (state != M_NB) {
+	    state = M_NB;
+	    reset_scroll_speed();
+	    render_text("no batteries");
+	}
+
+	return;
+    }
+
     /* battery not present case */
     if (!binfo->present) {
 	if (state != M_NP) {
@@ -517,12 +553,6 @@ static void set_message(global_t *globals)
 		scroll_faster(0.75);
 		render_text("critical low battery");
 	    }
-	} else if (binfo->state == HARD_CRIT) {
-	    if (state != M_HCB) {
-		state = M_HCB;
-		scroll_faster(0.5);
-		render_text("hard critical low battery");
-	    }
 	} else if (binfo->state == LOW) {
 	    if (state != M_LB) {
 		state = M_LB;
@@ -536,11 +566,16 @@ static void set_message(global_t *globals)
 		render_text("on battery");
 	    }
 	}
-    }    
+    }
 }
 
 void set_time_display(global_t *globals)
 {
+    if (globals->battery_count == 0) {
+        invalid_time_display();
+	return;
+    }
+
     if (globals->binfo->charge_state == CHARGE)
 	display_time(globals->binfo->charge_time);
     else if (globals->binfo->charge_state == DISCHARGE)
@@ -549,51 +584,31 @@ void set_time_display(global_t *globals)
 	invalid_time_display();
 }
 
+void clear_batt_id_area(void)
+{
+    copy_xpm_area(125, 40, 7, 11, 51, 32);
+}
+
 void set_batt_id_area(int bno)
 {
     int w = 7;			/* Width of the number */
     int h = 11;			/* Height of the number */
     int dx = 50;		/* x coord of the target area */
-    int dy = 31;		/* y coord of the target area */
+    int dy = 32;		/* y coord of the target area */
     int sx = (bno + 1) * 7;	/* source x coord */
     int sy = 76;		/* source y coord */
-    
+
     copy_xpm_area(sx, sy, w, h, dx, dy);
 }
 
-void usage(char *name)
-{
-    printf("%s - help\t\t[simon@dreamcraft.com.au]\n\n"
-	   "-d display\t\tdisplay on remote display <display>\n"
-	   "-b\t\t\tenable blinking of various UI elements\n"
-	   "-r\t\t\tdisable scrolling message\n"
-	   "-c value\t\tset critical low alarm at <value> percent\n"
-	   "\t\t\t(default: 10 percent)\n"
-	   "-m <battery number>\tbattery number to monitor\n"
-	   "-s <sample rate>\tnumber of times per minute to sample battery information\n"
-	   "\t\t\tdefault 20 (once every three seconds)\n"
-	   "-f\t\t\tforce the use of capacity mode for calculating time remaining\n"
-	   "-n\t\t\tdo not blink\n"
-	   "-w\t\t\trun in command line mode\n"
-	   "-a <samples>\t\tsamples to average over (cli mode only)\n"
-	   "-v\t\t\tincrease verbosity\n"
-	   "\t\t\tcan be used multiple times to increase verbosity further\n"
-	   "-h\t\t\tdisplay this help\n",
-	   name);
-}
-
-void print_version(void)
-{
-    printf("wmacpi version %s\n", WMACPI_VER);
-    printf(" Using libacpi version %s\n", LIBACPI_VER);
-}
+#define VERSION "wmacpi version " WMACPI_VER "\nUsing libacpi version " LIBACPI_VER
 
 void cli_wmacpi(global_t *globals, int samples)
 {
     int i, j, sleep_time = 0;
     battery_t *binfo;
     adapter_t *ap;
-    
+
     pdebug("samples: %d\n", samples);
     if(samples > 1)
     	sleep_time = 1000000/samples;
@@ -605,7 +620,7 @@ void cli_wmacpi(global_t *globals, int samples)
 	acquire_global_info(globals);
 	usleep(sleep_time);
     }
-    
+
     ap = &globals->adapter;
     if(ap->power == AC) {
 	printf("On AC Power");
@@ -614,8 +629,8 @@ void cli_wmacpi(global_t *globals, int samples)
 	    if(binfo->present && (binfo->charge_state == CHARGE)) {
 		printf("; Battery %s charging", binfo->name);
 		printf(", currently at %2d%%", binfo->percentage);
-		if(binfo->charge_time >= 0) 
-		    printf(", %2d:%02d remaining", 
+		if(binfo->charge_time >= 0)
+		    printf(", %2d:%02d remaining",
 			   binfo->charge_time/60,
 			   binfo->charge_time%60);
 	    }
@@ -630,31 +645,56 @@ void cli_wmacpi(global_t *globals, int samples)
 		       binfo->percentage);
 	}
 	if(globals->rtime >= 0)
-	    printf("; %d:%02d remaining", globals->rtime/60, 
+	    printf("; %d:%02d remaining", globals->rtime/60,
 		   globals->rtime%60);
 	printf("\n");
     }
     return;
 }
 
+battery_t *switch_battery(global_t *globals, int battno)
+{
+  globals->binfo = &batteries[battno];
+  pinfo("changing to monitor battery %s\n", globals->binfo->name);
+  set_batt_id_area(battno);
+  dockapp->update = 1;
+
+  return globals->binfo;
+}
+
+
 int main(int argc, char **argv)
 {
     char *display = NULL;
-    int ch;
     int sample_count = 0;
     int batt_reinit, ac_reinit;
     int batt_count = 0;
     int ac_count = 0;
-    int cli = 0, samples = 1;
+    int cli = 0, samples = 1, critical = 10;
     int samplerate = 20;
-    int sleep_rate = 10;
-    int sleep_time = 1000000/sleep_rate;
     int scroll_count = 0;
     enum rtime_mode rt_mode = RT_RATE;
     int rt_forced = 0;
-    battery_t *binfo;
+    battery_t *binfo = NULL;
     global_t *globals;
 
+    fd_set fds;
+    struct timeval tv_rate;
+    struct timeval tv = {0, 0};
+
+    DAProgramOption options[] = {
+     {"-r", "--no-scroll", "disable scrolling message", DONone, False, {NULL}},
+     {"-n", "--no-blink", "disable blinking of various UI elements", DONone, False, {NULL}},
+     {"-x", "--cmdline", "run in command line mode",  DONone, False, {NULL}},
+     {"-f", "--force-capacity-mode", "force the use of capacity mode for calculating time remaining", DONone, False, {NULL}},
+     {"-d", "--display", "display or remote display", DOString, False, {&display}},
+     {"-c", "--critical", "set critical low alarm at <number> percent\n                               (default: 10 percent)", DONatural, False, {&critical}},
+     {"-m", "--battery", "battery number to monitor", DONatural, False, {&battery_no}},
+     {"-s", "--sample-rate", "number of times per minute to sample battery information\n                               default 20 (once every three seconds)", DONatural, False, {&samplerate}},
+     {"-V", "--verbosity", "Set verbosity", DONatural, False, {&verbosity}},
+     {"-a", "--samples", "number of samples to average over (cli mode only)",  DONatural, False, {&samples}},
+    };
+
     dockapp = calloc(1, sizeof(struct dockapp));
     globals = calloc(1, sizeof(global_t));
 
@@ -665,8 +705,8 @@ int main(int argc, char **argv)
     globals->crit_level = 10;
     battery_no = 1;
 
-    /* after this many samples, we reinit the battery and AC adapter 
-     * information. 
+    /* after this many samples, we reinit the battery and AC adapter
+     * information.
      * XXX: make these configurable . . . */
     batt_reinit = 100;
     ac_reinit = 1000;
@@ -675,84 +715,64 @@ int main(int argc, char **argv)
      * are available /before/ we can decide if the battery we want to
      * monitor is available. */
     /* parse command-line options */
-    while ((ch = getopt(argc, argv, "d:c:m:s:a:fhnwbrvV")) != EOF) {
-	switch (ch) {
-	case 'c':
-	    if (optarg) {
-		globals->crit_level = atoi(optarg);
-		if ((globals->crit_level < 0) || (globals->crit_level > 100)) {
-		    fprintf(stderr, "Please use values between 0 and 100%%\n");
-		    globals->crit_level = 10;
-		    fprintf(stderr, "Using default value of 10%%\n");
-		}
-	    }
-	    break;
-	case 'd':
-	    if (optarg)
-		display = strdup(optarg);
-	    break;
-	case 'm':
-	    if (optarg) {
-		battery_no = atoi(optarg);
-		if (battery_no >= MAXBATT) {
-		    fprintf(stderr, "Please specify a battery number below %d\n",
-			    MAXBATT);
-		    return 1;
-		}
-		pinfo("Monitoring battery %d\n", battery_no);
-	    } 
-	    break;
-	case 's':
-	    if (optarg) {
-		samplerate = atoi(optarg);
-		if (samplerate == 0) samplerate = 1;
-		if (samplerate > 600) samplerate = 600;
-	    } else {
-		usage(argv[0]);
-		exit(1);
-	    }
-	    break;
-	case 'f':
-	    rt_mode = RT_CAP;
-	    rt_forced = 1;
-	    break;
-	case 'h':
-	    usage(argv[0]);
-	    return 0;
-	case 'v':
-	    verbosity++;
-	    break;
-	case 'V':
-	    print_version();
-	    return 0;
-	case 'n':
-	    dockapp->blink = 0;
-	    break;
-	case 'w':
-	    cli = 1;
-	    break;
-	case 'a':
-	    if(optarg != NULL) {
-		samples = atoi(optarg);
-		if(samples > 1000 || samples <= 0) {
-		    fprintf(stderr, "Please specify a reasonable number of samples\n");
-		    exit(1);
-		}
-	    }
-	    break;
-	case 'b':
-	    dockapp->blink = 1;
-	    break;
-	case 'r':
-	    dockapp->scroll = 0;
-	    break;
-	default:
-	    usage(argv[0]);
-	    return 1;
-	}
-	
+    DAParseArguments(argc, argv, options, 10,
+      "A battery monitor dockapp for ACPI based systems",
+      VERSION);
+
+    if (options[0].used)
+        dockapp->scroll = 0;
+    if (options[1].used)
+        dockapp->blink = 0;
+    if (options[2].used)
+        cli = 1;
+    if (options[3].used) {
+        rt_mode = RT_CAP;
+        rt_forced = 1;
     }
-    
+
+    if (samplerate == 0) samplerate = 1;
+    if (samplerate > 600) samplerate = 600;
+
+    /* convert to number of base periods */
+    samplerate = ((60 * 1000000) / samplerate) / BASE_PERIOD;
+
+    if (!dockapp->scroll) {
+	if (!dockapp->blink) {
+	    /* Adapt the period to the sample rate */
+	    tv_rate.tv_usec = samplerate * BASE_PERIOD;
+
+	    tv_rate.tv_sec = tv_rate.tv_usec / 1000000;
+	    tv_rate.tv_usec = tv_rate.tv_usec - (tv_rate.tv_sec * 1000000);
+
+	    dockapp->period_length = samplerate;
+	} else {
+	    /* blinking is every second */
+	    tv_rate.tv_sec = 1; /* BASE_PERIOD * 10 = 1 sec */
+	    tv_rate.tv_usec = 0;
+
+	    dockapp->period_length = 10;
+	}
+    } else {
+	/* scrolling is every BASE_PERIOD (100 ms) */
+	tv_rate.tv_sec = 0;
+	tv_rate.tv_usec = BASE_PERIOD;
+
+	dockapp->period_length = 1;
+    }
+
+    if (critical > 100) {
+        fprintf(stderr, "Please use values between 0 and 100%%\n");
+        fprintf(stderr, "Using default value of 10%%\n");
+        critical = 10;
+    }
+    globals->crit_level = critical;
+
+    if (battery_no >= MAXBATT) {
+        fprintf(stderr, "Please specify a battery number below %d\n", MAXBATT);
+        return 1;
+    }
+    pinfo("Monitoring battery %d\n", battery_no);
+
     if (power_init(globals))
 	/* power_init functions handle printing error messages */
 	exit(1);
@@ -761,8 +781,7 @@ int main(int argc, char **argv)
     globals->rt_forced = rt_forced;
 
     if (battery_no > globals->battery_count) {
-	pfatal("Battery %d not available for monitoring.\n", battery_no);
-	exit(1);
+	pinfo("Battery %d not available for monitoring.\n", battery_no);
     }
 
     /* check for cli mode */
@@ -781,23 +800,29 @@ int main(int argc, char **argv)
     battery_no--;
 
     /* make new dockapp window */
-    /* Don't even /think/ of asking me why, but if I set the window name to 
+    /* Don't even /think/ of asking me why, but if I set the window name to
      * "acpi", the app refuses to dock properly - it's just plain /weird/.
      * So, wmacpi it is . . . */
     new_window(display, "wmacpi", argc, argv);
 
     /* get initial statistics */
     acquire_all_info(globals);
-    binfo = &batteries[battery_no];
-    globals->binfo = binfo;
-    pinfo("monitoring battery %s\n", binfo->name);
+
+    if (globals->battery_count > 0) {
+	binfo = &batteries[battery_no];
+	globals->binfo = binfo;
+	set_batt_id_area(battery_no);
+	pinfo("monitoring battery %s\n", binfo->name);
+    }
+
     clear_time_display();
     set_power_panel(globals);
     set_message(globals);
-    set_batt_id_area(battery_no);
 
     /* main loop */
     while (1) {
+	Atom atom;
+	Atom wmdelwin;
 	XEvent event;
 	while (XPending(dockapp->display)) {
 	    XNextEvent(dockapp->display, &event);
@@ -815,60 +840,98 @@ int main(int argc, char **argv)
 	    case ButtonPress:
 		break;
 	    case ButtonRelease:
+		if (globals->battery_count == 0)
+		    break;
+
 		/* cycle through the known batteries. */
 		battery_no++;
 		battery_no = battery_no % globals->battery_count;
-		globals->binfo = &batteries[battery_no];
-		binfo = globals->binfo;
-		pinfo("changing to monitor battery %s\n", binfo->name);
-		set_batt_id_area(battery_no);
-		dockapp->update = 1;
+
+		binfo = switch_battery(globals, battery_no);
+		break;
+	    case ClientMessage:
+		/* what /is/ this crap?
+		 * Turns out that libdockapp adds the WM_DELETE_WINDOW atom to
+		 * the WM_PROTOCOLS property for the window, which means that
+		 * rather than get a simple DestroyNotify message, we get a
+		 * nice little message from the WM saying "hey, can you delete
+		 * yourself, pretty please?". So, when running as a window
+		 * rather than an icon, we're impossible to kill in a friendly
+		 * manner, because we're expecting to die from a DestroyNotify
+		 * and thus blithely ignoring the WM knocking on our window
+		 * border . . .
+		 *
+		 * This simply checks for that scenario - it may fail oddly if
+		 * something else comes to us via a WM_PROTOCOLS ClientMessage
+		 * event, but I suspect it's not going to be an issue. */
+		wmdelwin = XInternAtom(dockapp->display, "WM_DELETE_WINDOW", 1);
+		atom = event.xclient.data.l[0];
+		if (atom == wmdelwin) {
+		    XCloseDisplay(dockapp->display);
+		    exit(0);
+		}
 		break;
 	    }
 	}
 
 	/* XXX: some laptops have problems with sampling the battery
 	 * regularly - apparently, the BIOS disables interrupts while
-	 * reading from the battery, which is generally on a slow bus 
+	 * reading from the battery, which is generally on a slow bus
 	 * and is a slow device, so you get significant periods without
-	 * interrupts. This causes interactivity to suffer . . . 
-	 * 
-	 * My proposed workaround is to allow the user to set the sample
-	 * rate - it defaults to ten, but can be set lower (or higher).
+	 * interrupts. This causes interactivity to suffer . . .
 	 *
-	 * The only problem with this is that we need to sample less 
-	 * frequently, while still allowing the app to update normally. 
-	 * That means calling redraw_window() and all the set_*() functions
-	 * normally, but only calling acquire_all_info() every so often. 
-	 * As it stands, we only call acquire_all_info() once every three
-	 * seconds (once every thirty updates) . . . I'm not entirely sure
-	 * /how/ this could cause interactivity problems, but hey . . . 
+	 * So, the workaround/fix for this is to sample at a much
+	 * lower rate than we may update/refresh/expose/whatever. The
+	 * user specifies how many times they want us to sample per
+	 * minute; we use select() on our X events fd to wake up when
+	 * there's some display work to be done, with the timeout set
+	 * to whatever the time between samples is. When we hit our
+	 * select() timeout we update our samples, otherwise we update
+	 * the display.
 	 *
-	 * So, given the base rate of once every three seconds, we want to
-	 * change this test to . . . */
-	/* Okay, this needs /fixing/ - it's ridiculous. We should be giving
-	 * the user the option of saying how many times per minute the 
-	 * battery should be sampled, defaulting to 20 times. 
-	 * 
-	 * We sleep for one tenth of a second at a time, so 60 seconds
-	 * translates to 600 sleeps. So, we change the default sample
-	 * rate to 20, and the calculation below becomes . . .*/
-	if (sample_count++ == ((sleep_rate*60)/samplerate)) {
+	 * Note that this has a wrinkle when blinking and/or scrolling
+	 * is enabled, since we need to update the display more
+	 * frequently than we sample (most likely). In that case we
+	 * set the timeout based on the display update cycle. */
+
+	/* have we completed our timeout, or were we woken up early? */
+	if ((tv.tv_sec != 0) || (tv.tv_usec != 0))
+	    goto win_update;
+
+	tv = tv_rate;
+
+	sample_count += dockapp->period_length;
+
+	if (sample_count >= samplerate) {
+	    if (globals->battery_count == 0) {
+	        batt_count = 0;
+
+	        reinit_batteries(globals);
+
+		/* battery appeared */
+		if (globals->battery_count > 0) {
+		    if (battery_no > globals->battery_count)
+		        battery_no = 0;
+
+		    binfo = switch_battery(globals, battery_no);
+		}
+	    }
+
 	    acquire_all_info(globals);
 
 	    /* we need to be able to reinitialise batteries and adapters, because
 	     * they change - you can hotplug batteries on most laptops these days
 	     * and who knows what kind of shit will be happening soon . . . */
 	    if (batt_count++ >= batt_reinit) {
-		    if(reinit_batteries(globals)) 
-			    pfatal("Oh my god, the batteries are gone!\n");
-		    batt_count = 0;
+		if(reinit_batteries(globals))
+		    pfatal("Oh my god, the batteries are gone!\n");
+		batt_count = 0;
 	    }
 
 	    if (ac_count++ >= ac_reinit) {
-		    if(reinit_ac_adapters(globals)) 
-			    pfatal("What happened to our AC adapters?!?\n");
-		    ac_count = 0;
+		if(reinit_ac_adapters(globals))
+		    pfatal("What happened to our AC adapters?!?\n");
+		ac_count = 0;
 	    }
 	    sample_count = 0;
 	}
@@ -878,28 +941,36 @@ int main(int argc, char **argv)
 	    scroll_count = 0;
 	}
 
-	/* The old code had some kind of weird crap with timers and the like. 
+	/* The old code had some kind of weird crap with timers and the like.
 	 * As far as I can tell, it's meaningless - the time we want to display
-	 * is the time calculated from the remaining capacity, as per the 
+	 * is the time calculated from the remaining capacity, as per the
 	 * ACPI spec. The only thing I'd change is the handling of a charging
-	 * state: my best guess, based on the behaviour I'm seeing with my 
+	 * state: my best guess, based on the behaviour I'm seeing with my
 	 * Lifebook, is that the present rate value when charging is the rate
 	 * at which the batteries are being charged, which would mean I'd just
-	 * need to reverse the rtime calculation to be able to work out how 
-	 * much time remained until the batteries were fully charged . . . 
+	 * need to reverse the rtime calculation to be able to work out how
+	 * much time remained until the batteries were fully charged . . .
 	 * That would be rather useful, though given it would vary rather a lot
 	 * it seems likely that it'd be little more than a rough guesstimate. */
 	set_time_display(globals);
 	set_power_panel(globals);
 	set_message(globals);
-	display_percentage(binfo->percentage);
+
+	if (globals->battery_count == 0) {
+	    clear_percentage();
+	    clear_batt_id_area();
+	} else
+	    display_percentage(binfo->percentage);
+
 	scroll_text();
 
-	/* redraw_window, if anything changed - determined inside 
-	 * redraw_window. */
+    win_update:
+	/* redraw_window, if anything changed */
 	redraw_window();
 
-	usleep(sleep_time);
+	FD_ZERO(&fds);
+	FD_SET(dockapp->x_fd, &fds);
+	select(FD_SETSIZE, &fds, NULL, NULL, &tv);
     }
     return 0;
 }
